/* last modified 09/16/05 */
/*
 *******************************************************************************
 *                                                                             *
 *   EvaluatePieces() is used to evaluate all pieces other than pawns, but     *
 *   does not include some piece-related things like development and king-     *
 *   safety as it relates to castling.                                         *
 *                                                                             *
 *******************************************************************************
 */
int EvaluateBishops(TREE * RESTRICT tree)
{
  register BITBOARD temp;
  register int square, file, tscore;
  register int score=0;
/*
 **********************************************************************
 *                                                                    *
 *   bishop evaluation includes mobility, centralization as well as   *
 *   a bonus for having the bishop pair.  a special case is a bishop  *
 *   that is fianchettoed in front of a castled king.  this bishop is *
 *   defending very weak squares and is therefore more valuable and   *
 *   should not be traded.                                            *
 *                                                                    *
 **********************************************************************
 */
/*
 ************************************************************
 *                                                          *
 *   white bishops                                          *
 *                                                          *
 ************************************************************
 */
/*
 ************************************************************
 *                                                          *
 *   check to see if the bishop is trapped at a2 or h2 with *
 *   a pawn at b2 or g2 that can advance one square and     *
 *   trap the bishop, or a pawn at b3 or g3 that has        *
 *   trapped the bishop already.  also test for the bishop  *
 *   at b1 or g1 as that might not be an escape.            *
 *                                                          *
 ************************************************************
 */
  if (WhiteBishops) {
    if (WhiteBishops & mask_WBT) {
      if (WhiteBishops & SetMask(A7) && SetMask(B6) & BlackPawns)
        score -= bishop_trapped;
      else if (WhiteBishops & SetMask(B8) && SetMask(C7) & BlackPawns)
        score -= bishop_trapped;
      else if (WhiteBishops & SetMask(H7) && SetMask(G6) & BlackPawns)
        score -= bishop_trapped;
      else if (WhiteBishops & SetMask(G8) && SetMask(F7) & BlackPawns)
        score -= bishop_trapped;
    }
/*
 ************************************************************
 *   first, locate each bishop and add in its static score  *
 *   from the bishop piece/square table.                    *
 *                                                          *
 ************************************************************
 */
    temp = WhiteBishops;
    while (temp) {
      square = FirstOne(temp);
      score += bval_w[square];
/*
 ************************************************************
 *                                                          *
 *   then fold in the mobility score.                       *
 *                                                          *
 ************************************************************
 */
      score += bishop_mobility * MobilityBishop(square);
/*
 ************************************************************
 *                                                          *
 *   now add in a bonus for a bishop blocking a center pawn *
 *   at D6/E6 as that is very cramping.                     *
 *                                                          *
 ************************************************************
 */
      if (white_outpost[square] && !(mask_no_pattacks_b[square] & BlackPawns)) {
        if ((square == D6 || square == E6) && PcOnSq(square + 8) == -pawn)
          score += blocked_center_pawn;
      }
/*
 ************************************************************
 *                                                          *
 *   adjust the white tropism count for this piece.         *
 *                                                          *
 ************************************************************
 */
      tree->w_tropism += king_tropism_b[Distance(square, tree->b_kingsq)];
      Clear(square, temp);
    }
  }
#ifdef DEBUGEV
  printf("score[bishops(white)]=            %4d\n", score);
#endif
/*
 ************************************************************
 *                                                          *
 *   black bishops                                          *
 *                                                          *
 ************************************************************
 */
/*
 ************************************************************
 *                                                          *
 *   check to see if the bishop is trapped at a7 or h7 with *
 *   a pawn at b7 or g7 that can advance one square and     *
 *   trap the bishop, or a pawn at b6 or g6 that has        *
 *   trapped the bishop already.  also test for the bishop  *
 *   at b8 or g8 as that might not be an escape.            *
 *                                                          *
 ************************************************************
 */
  if (BlackBishops) {
    if (BlackBishops & mask_BBT) {
      if (BlackBishops & SetMask(A2) && SetMask(B3) & WhitePawns)
        score += bishop_trapped;
      else if (BlackBishops & SetMask(B1) && SetMask(C2) & WhitePawns)
        score += bishop_trapped;
      else if (BlackBishops & SetMask(H2) && SetMask(G3) & WhitePawns)
        score += bishop_trapped;
      else if (BlackBishops & SetMask(G1) && SetMask(F2) & WhitePawns)
        score += bishop_trapped;
    }
/*
 ************************************************************
 *                                                          *
 *   first, locate each bishop and add in its static score  *
 *   from the bishop piece/square table.                    *
 *                                                          *
 ************************************************************
 */
    temp = BlackBishops;
    while (temp) {
      square = FirstOne(temp);
      score -= bval_b[square];
/*
 ************************************************************
 *                                                          *
 *   then fold in the mobility score.                       *
 *                                                          *
 ************************************************************
 */
      score -= bishop_mobility * MobilityBishop(square);
/*
 ************************************************************
 *                                                          *
 *   now add in a bonus for a bishop blocking a center pawn *
 *   at D3/E3 as that is very cramping.                     *
 *                                                          *
 ************************************************************
 */
      if (black_outpost[square] && !(mask_no_pattacks_w[square] & WhitePawns)) {
        if ((square == D3 || square == E3) && PcOnSq(square - 8) == pawn)
          score -= blocked_center_pawn;
      }
/*
 ************************************************************
 *                                                          *
 *   adjust the black tropism count for this piece.         *
 *                                                          *
 ************************************************************
 */
      tree->b_tropism += king_tropism_b[Distance(square, tree->w_kingsq)];
      Clear(square, temp);
    }
  }
#ifdef DEBUGEV
  printf("score[bishops(black)]=            %4d\n", score);
#endif
  return (score);
}
