/* last modified 09/16/05 */
/*
 *******************************************************************************
 *                                                                             *
 *   EvaluatePieces() is used to evaluate all pieces other than pawns, but     *
 *   does not include some piece-related things like development and king-     *
 *   safety as it relates to castling.                                         *
 *                                                                             *
 *******************************************************************************
 */
int EvaluateQueens(TREE * RESTRICT tree)
{
  register BITBOARD temp;
  register int square, file, tscore;
  register int score=0;
/*
 **********************************************************************
 *                                                                    *
 *   queen evaluation includes centralization, plus some bonuses that *
 *   affect the opponent's king safety.                               *
 *                                                                    *
 **********************************************************************
 */
/*
 ************************************************************
 *                                                          *
 *   white queens                                           *
 *                                                          *
 *   first locate each queen and obtain it's centralization *
 *   score from the static piece/square table for queens.   *
 *                                                          *
 *   then, if the opposing side's king safety is much worse *
 *   than the king safety for this side, add in a bonus to  *
 *   keep the queen around.                                 *
 *                                                          *
 ************************************************************
 */
  temp = WhiteQueens;
  while (temp) {
    square = FirstOne(temp);
    score += qval_w[square];
/*
 ************************************************************
 *                                                          *
 *   check to see if the queen is in a strong position on   *
 *   the 7th rank supported by a rook on the 7th.  if so,   *
 *   the positional advantage is almost overwhelming.       *
 *                                                          *
 ************************************************************
 */
    if (Rank(square) == RANK7 && (BlackPawns & rank_mask[RANK7] ||
            (BlackKingSQ > H7))) {
      if (AttacksRank(square) & WhiteRooks)
        score += queen_rook_on_7th_rank;
    }
/*
 ************************************************************
 *                                                          *
 *   adjust the white tropism count for this piece.         *
 *                                                          *
 ************************************************************
 */
    trop = 7;
    if (!(WhitePawns & plus8dir[square]))
      trop = FileDistance(square, tree->b_kingsq);
    tree->w_tropism +=
        Max(king_tropism_q[Distance(square, tree->b_kingsq)],
        king_tropism_at_q[trop]);
/*
 ************************************************************
 *                                                          *
 *   add in a bonus if the opponent's king position is not  *
 *   safe, since the queen is a strong attacking piece.     *
 *                                                          *
 ************************************************************
 */
    if (tree->b_safety > tree->w_safety + 4)
      score += queen_is_strong;
/*
 ************************************************************
 *                                                          *
 *   add in a penalty for being on the wrong side of the    *
 *   board, which can lead to attacks by the opponent.      *
 *                                                          *
 ************************************************************
 */
    if (TotalWhitePawns > 4) {
      int offside = (File(square) < FILEC && File(tree->b_kingsq) > FILEE)
          || (File(square) > FILEF && File(tree->b_kingsq) < FILED);

      if (offside) {
        tree->w_tropism -= queen_offside_tropism;
        tree->w_tropism = Max(tree->w_tropism, -8);
      }
    }
    Clear(square, temp);
  }
#ifdef DEBUGEV
  printf("score[queens(white)]=             %4d\n", score);
#endif
/*
 ************************************************************
 *                                                          *
 *   black queens                                           *
 *                                                          *
 *   first locate each queen and obtain it's centralization *
 *   score from the static piece/square table for queens.   *
 *                                                          *
 *   then, if the opposing side's king safety is much worse *
 *   than the king safety for this side, add in a bonus to  *
 *   keep the queen around.                                 *
 *                                                          *
 ************************************************************
 */
  temp = BlackQueens;
  while (temp) {
    square = FirstOne(temp);
    score -= qval_b[square];
/*
 ************************************************************
 *                                                          *
 *   check to see if the queen is in a strong position on   *
 *   the 7th rank supported by a rook on the 7th.  if so,   *
 *   the positional advantage is almost overwhelming.       *
 *                                                          *
 ************************************************************
 */
    if (Rank(square) == RANK2 && (WhitePawns & rank_mask[RANK2] ||
            (WhiteKingSQ < A2))) {
      if (AttacksRank(square) & BlackRooks)
        score -= queen_rook_on_7th_rank;
    }
/*
 ************************************************************
 *                                                          *
 *   adjust the black tropism count for this piece.         *
 *                                                          *
 ************************************************************
 */
    trop = 7;
    if (!(BlackPawns & minus8dir[square]))
      trop = FileDistance(square, tree->w_kingsq);
    tree->b_tropism +=
        Max(king_tropism_q[Distance(square, tree->w_kingsq)],
        king_tropism_at_q[trop]);
/*
 ************************************************************
 *                                                          *
 *   add in a bonus if the opponent's king position is not  *
 *   safe, since the queen is a strong attacking piece.     *
 *                                                          *
 ************************************************************
 */
    if (tree->w_safety > tree->b_safety + 4)
      score -= queen_is_strong;
/*
 ************************************************************
 *                                                          *
 *   add in a penalty for being on the wrong side of the    *
 *   board, which can lead to attacks by the opponent.      *
 *                                                          *
 ************************************************************
 */
    if (TotalBlackPawns > 4) {
      int offside = (File(square) < FILEC && File(tree->w_kingsq) > FILEE)
          || (File(square) > FILEF && File(tree->w_kingsq) < FILED);

      if (offside) {
        tree->b_tropism -= queen_offside_tropism;
        tree->b_tropism = Max(tree->b_tropism, -8);
      }
    }
    Clear(square, temp);
  }
#ifdef DEBUGEV
  printf("score[queens(black)]=             %4d\n", score);
#endif
  return (score);
}
