/* last modified 09/16/05 */
/*
 *******************************************************************************
 *                                                                             *
 *   EvaluatePieces() is used to evaluate all pieces other than pawns, but     *
 *   does not include some piece-related things like development and king-     *
 *   safety as it relates to castling.                                         *
 *                                                                             *
 *******************************************************************************
 */
int EvaluateRooks(TREE * RESTRICT tree)
{
  register BITBOARD temp;
  register int square, file, tscore;
  register int score=0;
/*
 ************************************************************
 *                                                          *
 *   white rooks                                            *
 *                                                          *
 ************************************************************
 */
  temp = WhiteRooks;
  while (temp) {
    register int mobility;

    square = FirstOne(temp);
    file = File(square);
    score += rval_w[square];
/*
 ************************************************************
 *                                                          *
 *   determine if the rook is on an open file.  if it is,   *
 *   determine if this rook attacks another friendly rook,  *
 *   making it difficult to drive the rooks off the file.   *
 *                                                          *
 ************************************************************
 */
    trop = 7;
    if (!(file_mask[file] & tree->all_pawns)) {
      score += rook_open_file[file] * MobilityFile(square);
      trop = FileDistance(square, tree->b_kingsq);
    } else {
      if (tree->pawn_score.open_files) {
        unsigned char rankmvs = AttacksRank(square) >> (56 - (square & 0x38));

        if (!(rankmvs & tree->pawn_score.open_files))
          score -=
              (rook_open_file[FILED] >> 1) * MobilityFile((square & 0x38) +
              FILED);
      }
      if (!(file_mask[file] & WhitePawns)) {
        score += rook_half_open_file[file] * MobilityFile(square);
        trop = FileDistance(square, tree->b_kingsq);
      } else if (!(plus8dir[square] & WhitePawns)) {
        trop = FileDistance(square, tree->b_kingsq);
      }
    }
/*
 ************************************************************
 *                                                          *
 *   see if the rook can move horizontally.  if not, it is  *
 *   somewhat precarious in how it is situated.             *
 *                                                          *
 ************************************************************
 */
    mobility = 0;
    if (File(square) < FILEH && PcOnSq(square + 1) <= 0)
      mobility++;
    if (File(square) > FILEA && PcOnSq(square - 1) <= 0)
      mobility++;
    if (!mobility)
      score -= rook_limited;
/*
 ************************************************************
 *                                                          *
 *   see if the rook is behind a passed pawn.  if it is,    *
 *   it is given a bonus.                                   *
 *                                                          *
 ************************************************************
 */
    if (128 >> file & tree->pawn_score.passed_w) {
      register const int pawnsq = LastOne(WhitePawns & file_mask[file]);

      if (square < pawnsq)
          score += rook_behind_passed_pawn;
    }
    if (128 >> file & tree->pawn_score.passed_b) {
      register const int pawnsq = FirstOne(BlackPawns & file_mask[file]);

      if (pawnsq < square)
        score += rook_behind_passed_pawn;
    }
/*
 ************************************************************
 *                                                          *
 *   finally check to see if any rooks are on the 7th rank, *
 *   with the opponent having pawns on that rank and the    *
 *   opponent's king being hemmed in on the 7th/8th rank.   *
 *   if so, and another rook or queen is also on the 7th,   *
 *   then this is a *strong* positional advantage.          *
 *                                                          *
 ************************************************************
 */
    if (Rank(square) == RANK7 && (BlackKingSQ > H7 ||
            BlackPawns & rank_mask[RANK7])) {
      score += rook_on_7th;
      if (tree->pawn_score.passed_w && BlackKingSQ > H7 &&
          !(BlackPieces & mask_abs7_w))
        score += rook_absolute_7th;
      if (AttacksRank(square) & (WhiteRooks | WhiteQueens))
        score += rook_connected_7th_rank;
    }
/*
 ************************************************************
 *                                                          *
 *   adjust the white tropism count for this piece.         *
 *                                                          *
 ************************************************************
 */
    tree->w_tropism +=
        Max(king_tropism_at_r[trop], king_tropism_r[Distance(square,
                tree->b_kingsq)]);
    Clear(square, temp);
  }
#ifdef DEBUGEV
  printf("score[rooks(white)]=              %4d\n", score);
#endif
/*
 ************************************************************
 *                                                          *
 *   black rooks                                            *
 *                                                          *
 ************************************************************
 */
  temp = BlackRooks;
  while (temp) {
    register int mobility;

    square = FirstOne(temp);
    file = File(square);
    score -= rval_b[square];
/*
 ************************************************************
 *                                                          *
 *   determine if the rook is on an open file.  if it is,   *
 *   determine if this rook attacks another friendly rook,  *
 *   making it difficult to drive the rooks off the file.   *
 *                                                          *
 ************************************************************
 */
    trop = 7;
    if (!(file_mask[file] & tree->all_pawns)) {
      score -= rook_open_file[file] * MobilityFile(square);
      trop = FileDistance(square, tree->w_kingsq);
    } else {
      if (tree->pawn_score.open_files) {
        unsigned char rankmvs = AttacksRank(square) >> (56 - (square & 0x38));

        if (!(rankmvs & tree->pawn_score.open_files))
          score +=
              (rook_open_file[FILED] >> 1) * MobilityFile((square & 0x38) +
              FILED);
      }
      if (!(file_mask[file] & BlackPawns)) {
        score -= rook_half_open_file[file] * MobilityFile(square);
        trop = FileDistance(square, tree->w_kingsq);
      } else if (!(minus8dir[square] & BlackPawns)) {
        trop = FileDistance(square, tree->w_kingsq);
      }
    }
/*
 ************************************************************
 *                                                          *
 *   see if the rook can move horizontally.  if not, it is  *
 *   somewhat precarious in how it is situated.             *
 *                                                          *
 ************************************************************
 */
    mobility = 0;
    if (File(square) < FILEH && PcOnSq(square + 1) >= 0)
      mobility++;
    if (File(square) > FILEA && PcOnSq(square - 1) >= 0)
      mobility++;
    if (!mobility)
      score += rook_limited;
/*
 ************************************************************
 *                                                          *
 *   see if the rook is behind a passed pawn.  if it is,    *
 *   it is given a bonus.                                   *
 *                                                          *
 ************************************************************
 */
    if (128 >> file & tree->pawn_score.passed_b) {
      register const int pawnsq = FirstOne(BlackPawns & file_mask[file]);

      if (square > pawnsq)
          score -= rook_behind_passed_pawn;
    }
    if (128 >> file & tree->pawn_score.passed_w) {
      register const int pawnsq = LastOne(WhitePawns & file_mask[file]);

      if (pawnsq > square)
        score -= rook_behind_passed_pawn;
    }
/*
 ************************************************************
 *                                                          *
 *   finally check to see if any rooks are on the 7th rank, *
 *   with the opponent having pawns on that rank and the    *
 *   opponent's king being hemmed in on the 7th/8th rank.   *
 *   if so, and another rook or queen is also on the 7th,   *
 *   then this is a *strong* positional advantage.          *
 *                                                          *
 ************************************************************
 */
    if (Rank(square) == RANK2 && (WhiteKingSQ < A2 ||
            WhitePawns & rank_mask[RANK2])) {
      score -= rook_on_7th;
      if (tree->pawn_score.passed_b && WhiteKingSQ < A2 &&
          !(WhitePieces & mask_abs7_b))
        score -= rook_absolute_7th;
      if (AttacksRank(square) & (BlackRooks | BlackQueens))
        score -= rook_connected_7th_rank;
    }
/*
 ************************************************************
 *                                                          *
 *   adjust the black tropism count for this piece.         *
 *                                                          *
 ************************************************************
 */
    tree->b_tropism +=
        Max(king_tropism_at_r[trop], king_tropism_r[Distance(square,
                tree->w_kingsq)]);
    Clear(square, temp);
  }
#ifdef DEBUGEV
  printf("score[rooks(black)]=              %4d\n", score);
#endif
  return (score);
}
